package demo.quicktime;



import java.awt.BorderLayout;

import java.awt.event.ActionEvent;

import java.awt.event.ActionListener;

import java.awt.event.WindowAdapter;

import java.awt.event.WindowEvent;



import javax.swing.JButton;

import javax.swing.JComponent;

import javax.swing.JFileChooser;

import javax.swing.JFrame;

import javax.swing.JLabel;



import quicktime.QTSession;

import quicktime.app.view.MoviePlayer;

import quicktime.app.view.QTFactory;

import quicktime.io.OpenMovieFile;

import quicktime.io.QTFile;

import quicktime.std.movies.Movie;



/**

 * QTJViewer provides a demo application of how you can use QuickTime quickly and 

 * easily in a Java application.

 */

public class QTJViewer extends JFrame implements ActionListener

{



	private JButton btnPlay = null;

	private javax.swing.JPanel topPanel = null;

	private javax.swing.JPanel movieScreen = null;

	private javax.swing.JPanel controlPanel = null;

	private JButton btnStop = null;

	private JButton btnLoad = null;

	private javax.swing.JProgressBar movieProgress = null;

	private javax.swing.JButton btnPause = null;

	

	

	private Movie movie;

	private String movieName;

	private ProgressThread progressThread;

	

	

	private void initialize() 

	{

        this.setContentPane(getTopPanel());

        this.setSize(508, 386);			

	}

	

	public static void main(String[] args)

	{

		QTJViewer v = new QTJViewer();

	}

	

	public QTJViewer()

	{	

		initialize();

		setTitle("QTJViewer");

		setVisible(true);

		

		try

		{

			QTSession.open();

		}

		catch (Exception ex)

		{

			ex.printStackTrace();

		}

		

		addWindowListener(new WindowAdapter()

		{

			public void windowClosing(WindowEvent e)

			{

				if (progressThread != null)

					progressThread.end();

				QTSession.close();	

			}

			

			public void windowClosed(WindowEvent e)

			{

				System.exit(0);

			}

		}		

		);

	}

	

	/**

	 * Does the QuickTime specific code of loading the movies and then

	 * putting this movie onto a JComponent we can use in Swing.

	 * @param file the multimedia file name

	 * @return the so-called movie screen

	 */

	private JComponent getMovieScreen(String file)

	{

		try

		{

			QTFile qtFile = new QTFile(file);	

			OpenMovieFile openMovieFile = OpenMovieFile.asRead(qtFile);

			movie = Movie.fromFile(openMovieFile);

			MoviePlayer moviePlayer = new MoviePlayer(movie);

			return QTFactory.makeQTJComponent(moviePlayer).asJComponent();

		}

		catch (Exception ex)

		{

			ex.printStackTrace();

			return new JLabel("ERROR");

		}

	}

	

	public void actionPerformed(ActionEvent e)

	{

		try

		{

			// The Play button starts the movie

			if (e.getSource() == getBtnPlay())

			{

				movie.start();

			}

			// The Stop button stops the movie and moves the index back to the start

			else if (e.getSource() == getBtnStop())

			{

				movie.stop();

				movie.goToBeginning();

			}

			// The Pause button simply stops the movie

			else if (e.getSource() == getBtnPause())

			{

				movie.stop();

			}

			// The Load button loads the multimedia file.  It also starts

			// the ProgressThread, which tracks the progress of the movie

			// so we can know it's progress and react to the end of the movie

			// and update the progress bar as well.

			else if (e.getSource() == getBtnLoad())

			{				

				JFileChooser chooser = new JFileChooser();

				int returnVal = chooser.showOpenDialog(null);

				if (returnVal == JFileChooser.APPROVE_OPTION) 

				{

					if (progressThread != null)

					{

						progressThread.end();

					}

					movieName = chooser.getSelectedFile().getAbsolutePath();

					JComponent c = getMovieScreen(movieName);

					getMovieScreen().removeAll();

					getMovieScreen().add(c, BorderLayout.CENTER);

					getMovieScreen().revalidate();

					getMovieProgress().setMaximum(movie.getDuration());

					progressThread = new ProgressThread();

					Thread t = new Thread(progressThread);

					t.start();

				}

			}			

		}

		catch (Exception ex)

		{

			ex.printStackTrace();

		}

	}



	private JButton getBtnPlay() {

		if(btnPlay == null) {

			btnPlay = new JButton();

			btnPlay.setText("Play");

			btnPlay.setBackground(java.awt.Color.white);

			btnPlay.setBorderPainted(false);

			btnPlay.setFocusPainted(false);

			btnPlay.setFont(new java.awt.Font("SansSerif", java.awt.Font.BOLD, 10));

			btnPlay.setMargin(new java.awt.Insets(2,2,2,2));

			btnPlay.addActionListener(this);

		}

		return btnPlay;

	}



	private javax.swing.JPanel getTopPanel() {

		if(topPanel == null) {

			topPanel = new javax.swing.JPanel();

			java.awt.GridBagConstraints consGridBagConstraints7 = new java.awt.GridBagConstraints();

			java.awt.GridBagConstraints consGridBagConstraints6 = new java.awt.GridBagConstraints();

			consGridBagConstraints7.insets = new java.awt.Insets(4,13,7,11);

			consGridBagConstraints7.fill = java.awt.GridBagConstraints.BOTH;

			consGridBagConstraints7.weighty = 1.0;

			consGridBagConstraints7.weightx = 1.0;

			consGridBagConstraints7.gridy = 1;

			consGridBagConstraints7.gridx = 0;

			consGridBagConstraints6.insets = new java.awt.Insets(13,82,4,98);

			consGridBagConstraints6.ipady = 267;

			consGridBagConstraints6.ipadx = 310;

			consGridBagConstraints6.fill = java.awt.GridBagConstraints.BOTH;

			consGridBagConstraints6.weighty = 1.0;

			consGridBagConstraints6.weightx = 1.0;

			consGridBagConstraints6.gridy = 0;

			consGridBagConstraints6.gridx = 0;

			topPanel.setLayout(new java.awt.GridBagLayout());

			topPanel.add(getMovieScreen(), consGridBagConstraints6);

			topPanel.add(getControlPanel(), consGridBagConstraints7);

			topPanel.setBackground(java.awt.Color.white);

		}

		return topPanel;

	}



	private javax.swing.JPanel getMovieScreen() {

		if(movieScreen == null) {

			movieScreen = new javax.swing.JPanel();

			movieScreen.setBackground(java.awt.Color.WHITE);

		}

		return movieScreen;

	}



	private javax.swing.JPanel getControlPanel() {

		if(controlPanel == null) {

			controlPanel = new javax.swing.JPanel();

			java.awt.GridBagConstraints consGridBagConstraints1 = new java.awt.GridBagConstraints();

			java.awt.GridBagConstraints consGridBagConstraints2 = new java.awt.GridBagConstraints();

			java.awt.GridBagConstraints consGridBagConstraints3 = new java.awt.GridBagConstraints();

			java.awt.GridBagConstraints consGridBagConstraints5 = new java.awt.GridBagConstraints();

			java.awt.GridBagConstraints consGridBagConstraints4 = new java.awt.GridBagConstraints();

			consGridBagConstraints1.insets = new java.awt.Insets(17,6,10,4);

			consGridBagConstraints1.ipady = -4;

			consGridBagConstraints1.ipadx = 21;

			consGridBagConstraints1.gridy = 0;

			consGridBagConstraints1.gridx = 2;

			consGridBagConstraints2.insets = new java.awt.Insets(17,3,10,8);

			consGridBagConstraints2.ipady = -4;

			consGridBagConstraints2.ipadx = 20;

			consGridBagConstraints2.gridy = 0;

			consGridBagConstraints2.gridx = 4;

			consGridBagConstraints5.insets = new java.awt.Insets(17,4,10,2);

			consGridBagConstraints5.ipady = -4;

			consGridBagConstraints5.ipadx = 11;

			consGridBagConstraints5.gridy = 0;

			consGridBagConstraints5.gridx = 3;

			consGridBagConstraints3.insets = new java.awt.Insets(17,18,10,9);

			consGridBagConstraints3.ipady = -4;

			consGridBagConstraints3.ipadx = 18;

			consGridBagConstraints3.gridy = 0;

			consGridBagConstraints3.gridx = 0;

			consGridBagConstraints4.insets = new java.awt.Insets(20,9,13,5);

			consGridBagConstraints4.ipady = -5;

			consGridBagConstraints4.ipadx = 52;

			consGridBagConstraints4.fill = java.awt.GridBagConstraints.HORIZONTAL;

			consGridBagConstraints4.weightx = 1.0;

			consGridBagConstraints4.gridy = 0;

			consGridBagConstraints4.gridx = 1;

			controlPanel.setLayout(new java.awt.GridBagLayout());

			controlPanel.add(getBtnPlay(), consGridBagConstraints1);

			controlPanel.add(getBtnStop(), consGridBagConstraints2);

			controlPanel.add(getBtnLoad(), consGridBagConstraints3);

			controlPanel.add(getMovieProgress(), consGridBagConstraints4);

			controlPanel.add(getBtnPause(), consGridBagConstraints5);

			controlPanel.setBackground(java.awt.Color.white);

		}

		return controlPanel;

	}



	private JButton getBtnStop() {

		if(btnStop == null) {

			btnStop = new JButton();

			btnStop.setText("Stop");

			btnStop.setBorderPainted(false);

			btnStop.setFocusPainted(false);

			btnStop.setMargin(new java.awt.Insets(2,2,2,2));

			btnStop.setFont(new java.awt.Font("SansSerif", java.awt.Font.BOLD, 10));

			btnStop.setBackground(java.awt.Color.white);

			btnStop.addActionListener(this);

		}

		return btnStop;

	}



	private JButton getBtnLoad() {

		if(btnLoad == null) {

			btnLoad = new JButton();

			btnLoad.setText("Load");

			btnLoad.setBorderPainted(false);

			btnLoad.setBackground(java.awt.Color.white);

			btnLoad.setMargin(new java.awt.Insets(2,2,2,2));

			btnLoad.setFont(new java.awt.Font("SansSerif", java.awt.Font.BOLD, 10));

			btnLoad.setFocusPainted(false);

			btnLoad.addActionListener(this);

		}

		return btnLoad;

	}

	

	

	private javax.swing.JButton getBtnPause() {

		if(btnPause == null) {

			btnPause = new javax.swing.JButton();

			btnPause.setBackground(java.awt.Color.white);

			btnPause.setBorderPainted(false);

			btnPause.setFocusPainted(false);

			btnPause.setFont(new java.awt.Font("SansSerif", java.awt.Font.BOLD, 10));

			btnPause.setText("Pause");

			btnPause.setMargin(new java.awt.Insets(2,2,2,2));

			btnPause.addActionListener(this);

		}

		return btnPause;

	}



	private javax.swing.JProgressBar getMovieProgress() {

		if(movieProgress == null) {

			movieProgress = new javax.swing.JProgressBar();

			movieProgress.setStringPainted(true);

			movieProgress.setMinimum(0);

			movieProgress.setBackground(java.awt.Color.white);

			movieProgress.setForeground(java.awt.Color.black);

			movieProgress.setFont(new java.awt.Font("SansSerif", java.awt.Font.BOLD, 10));

			movieProgress.setBorderPainted(false);

		}

		return movieProgress;

	}

	

	private class ProgressThread implements Runnable

	{

		private volatile boolean keepRunning = true;

		

		public void run()

		{

			while (keepRunning)

			{

				try

				{

					// Continually gets the movie's current time

					// and updates the progress bar.

					// It also starts the movie over after it has completed.

					int time = movie.getTime();

					getMovieProgress().setValue(time);

					if (time == movie.getDuration())

					{

						movie.stop();

						Thread.sleep(1000);

						movie.goToBeginning();

					}

					Thread.sleep(300);

				}

				catch (Exception ex)

				{

					ex.printStackTrace();

				}

			}

		}

		

		public void end()

		{

			keepRunning = false;

		}

	}



}  